@@ -1,22 +1,23 @@
 # -*- coding: utf-8 -*-
 """API models for working with device and user assets."""
 import ipaddress
+from typing import Generator, List, Union
 
-from ..routers import API_VERSION
+from ..routers import API_VERSION, Router
 from .asset_mixin import AssetMixin
 
 
 class Devices(AssetMixin):
     """Device related API methods."""
 
-    FIELD_HOSTNAME = "specific_data.data.hostname"
-    FIELD_IP = "specific_data.data.network_interfaces.ips"
-    FIELD_IP_RAW = "specific_data.data.network_interfaces.ips_raw"
-    FIELD_MAC = "specific_data.data.network_interfaces.mac"
-    FIELD_SUBNET = "specific_data.data.network_interfaces.subnets"
+    FIELD_HOSTNAME: str = "specific_data.data.hostname"
+    FIELD_IP: str = "specific_data.data.network_interfaces.ips"
+    FIELD_IP_RAW: str = "specific_data.data.network_interfaces.ips_raw"
+    FIELD_MAC: str = "specific_data.data.network_interfaces.mac"
+    FIELD_SUBNET: str = "specific_data.data.network_interfaces.subnets"
 
     @property
-    def fields_default(self):
+    def fields_default(self) -> List[str]:
         """Fields to use by default for getting assets."""
         return self.FIELDS_API + [
             self.FIELD_HOSTNAME,
@@ -26,74 +27,94 @@ class Devices(AssetMixin):
         ]
 
     @property
-    def router(self):
+    def router(self) -> Router:
         """Router for this API model."""
         return API_VERSION.devices
 
-    def get_by_hostnames(self, values, **kwargs):
+    def get_by_hostnames(
+        self, values: List[str], **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where hostname in values."""
         kwargs["field"] = self.FIELD_HOSTNAME
         kwargs["field_manual"] = True
         kwargs["values"] = values
         return self.get_by_values(**kwargs)
 
-    def get_by_hostname_regex(self, value, **kwargs):
+    def get_by_hostname_regex(
+        self, value: str, **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where hostname regex matches value."""
         kwargs["field"] = self.FIELD_HOSTNAME
         kwargs["field_manual"] = True
         kwargs["value"] = value
         return self.get_by_value_regex(**kwargs)
 
-    def get_by_hostname(self, value, **kwargs):
+    def get_by_hostname(
+        self, value: str, **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where hostname == value."""
         kwargs["field"] = self.FIELD_HOSTNAME
         kwargs["field_manual"] = True
         kwargs["value"] = value
         return self.get_by_value(**kwargs)
 
-    def get_by_macs(self, values, **kwargs):
+    def get_by_macs(
+        self, values: List[str], **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where mac in values."""
         kwargs["field"] = self.FIELD_MAC
         kwargs["field_manual"] = True
         kwargs["values"] = values
         return self.get_by_values(**kwargs)
 
-    def get_by_mac_regex(self, value, **kwargs):
+    def get_by_mac_regex(
+        self, value: str, **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where mac regex matches value."""
         kwargs["field"] = self.FIELD_MAC
         kwargs["field_manual"] = True
         kwargs["value"] = value
         return self.get_by_value_regex(**kwargs)
 
-    def get_by_mac(self, value, **kwargs):
+    def get_by_mac(
+        self, value: str, **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where mac == value."""
         kwargs["field"] = self.FIELD_MAC
         kwargs["field_manual"] = True
         kwargs["value"] = value
         return self.get_by_value(**kwargs)
 
-    def get_by_ips(self, values, **kwargs):
+    def get_by_ips(
+        self, values: List[str], **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where ip in values."""
         kwargs["field"] = self.FIELD_IP
         kwargs["field_manual"] = True
         kwargs["values"] = values
         return self.get_by_values(**kwargs)
 
-    def get_by_ip_regex(self, value, **kwargs):
+    def get_by_ip_regex(
+        self, value: str, **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where ip regex matches value."""
         kwargs["field"] = self.FIELD_IP
         kwargs["field_manual"] = True
         kwargs["value"] = value
         return self.get_by_value_regex(**kwargs)
 
-    def get_by_ip(self, value, **kwargs):
+    def get_by_ip(
+        self, value: str, **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where ip == value."""
         kwargs["field"] = self.FIELD_IP
         kwargs["field_manual"] = True
         kwargs["value"] = value
         return self.get_by_value(**kwargs)
 
-    def get_by_subnet(self, value, not_flag=False, pre="", post="", **kwargs):
+    def get_by_subnet(
+        self, value: str, not_flag: bool = False, pre: str = "", post: str = "", **kwargs
+    ) -> Union[Generator[dict, None, None], List[dict]]:
         """Build a query to get assets where ip in subnet.
 
         Args:
