@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 """API models package."""
-from . import (central_core, discover, meta, nodes, roles, settings, system,
-               users)
+from . import central_core, discover, meta, nodes, roles, settings, system, users
 from .system import System
 
 __all__ = (
