@@ -1,17 +1,9 @@
 # -*- coding: utf-8 -*-
 """Command line interface for Axonius API Client."""
 from ..context import CONTEXT_SETTINGS, click
-from ..options import (
-    AUTH,
-    EXPORT,
-    FIELDS_SELECT,
-    PAGING,
-    QUERY,
-    add_options,
-    get_option_fields_default,
-    get_option_help,
-)
-from .grp_common import GET_EXPORT
+from ..options import (AUTH, EXPORT, FIELDS_SELECT, PAGING, QUERY, add_options,
+                       get_option_fields_default, get_option_help)
+from .grp_common import GET_EXPORT, load_whitelist
 
 OPTIONS = [
     *AUTH,
@@ -28,11 +20,13 @@ OPTIONS = [
 @click.command(name="get", context_settings=CONTEXT_SETTINGS)
 @add_options(OPTIONS)
 @click.pass_context
-def cmd(ctx, url, key, secret, query_file, **kwargs):
+def cmd(ctx, url, key, secret, query_file, whitelist=None, **kwargs):
     """Get assets using a query and fields."""
     if query_file:
         kwargs["query"] = query_file.read().strip()
 
+    kwargs["report_software_whitelist"] = load_whitelist(whitelist)
+
     client = ctx.obj.start_client(url=url, key=key, secret=secret)
 
     p_grp = ctx.parent.command.name
